/*
  @license
	Rollup.js v4.44.0
	Thu, 19 Jun 2025 06:21:58 GMT - commit fa4b2842c823f6a61f6b994a28b7fcb54419b6c6

	https://github.com/rollup/rollup

	Released under the MIT License.
*/
export { version as VERSION, defineConfig, rollup, watch } from './shared/node-entry.js';
import './shared/parseAst.js';
import '../native.js';
import 'node:path';
import 'path';
import 'node:process';
import 'node:perf_hooks';
import 'node:fs/promises';
