"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZigbeeEntity = void 0;
const entity_1 = __importDefault(require("./entity"));
class ZigbeeEntity extends entity_1.default {
}
exports.ZigbeeEntity = ZigbeeEntity;
//# sourceMappingURL=zigbeeEntity.js.map