export type Options = {
    trim?: boolean;
    wordWrap?: boolean;
    hard?: boolean;
};
export declare const wrap: (str: string, columns: number, options: Options) => string;
//# sourceMappingURL=index.d.ts.map