"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createAddUniqueNumber = void 0;
const createAddUniqueNumber = generateUniqueNumber => {
  return set => {
    const number = generateUniqueNumber(set);
    set.add(number);
    return number;
  };
};
exports.createAddUniqueNumber = createAddUniqueNumber;