import{u as p,j as n,F as d}from"./index-VJpwYvBU.js";import{s as x}from"./startCase-DNIChZAc.js";import{b as j}from"./index-C1LTvZEM.js";function h({children:t,feature:s,deviceValue:a,endpointSpecific:o}){const i=s.unit,[r,m]=j(s.name,a,i),{t:c}=p("zigbee"),e=s.name==="state"?s.property:s.name;return n.jsxs("div",{className:"flex flex-row items-center gap-1 mb-2",children:[n.jsx(d,{icon:r,className:m}),n.jsxs("div",{className:"grow-1",title:e,children:[x(e),!o&&n.jsx("span",{title:c(l=>l.endpoint),children:s.endpoint?` (${s.endpoint})`:null})]}),n.jsx("div",{className:"shrink-1 *:bg-base-200",children:t})]})}export{h as D};
