export declare const definitions: {
    zigbeeModel: string[];
    model: string;
    vendor: string;
    description: string;
    extend: import("../lib/types").ModernExtend[];
}[];
//# sourceMappingURL=quirky.d.ts.map