import type { Fz, KeyValue, ModernExtend, Tz } from "../lib/types";
import * as m from "./modernExtend";
export declare const ledvanceFz: {
    pbc_level_to_action: {
        cluster: "genLevelCtrl";
        type: ["commandMoveWithOnOff", "commandStopWithOnOff", "commandMove", "commandStop", "commandMoveToLevelWithOnOff"];
        convert: (model: import("../lib/types").Definition, msg: Fz.Message<"genLevelCtrl", undefined, ["commandMoveWithOnOff", "commandStopWithOnOff", "commandMove", "commandStop", "commandMoveToLevelWithOnOff"]>, publish: import("../lib/types").Publish, options: KeyValue, meta: Fz.Meta) => {
            [x: string]: unknown;
        };
    };
};
export declare const ledvanceTz: {
    ledvance_commands: {
        key: string[];
        convertSet: (entity: import("zigbee-herdsman/dist/controller/model").Group | import("zigbee-herdsman/dist/controller/model").Endpoint, key: string, value: unknown, meta: Tz.Meta) => Promise<void>;
    };
};
export declare function ledvanceOnOff(args?: m.OnOffArgs): ModernExtend;
export declare function ledvanceLight(args?: m.LightArgs): ModernExtend;
//# sourceMappingURL=ledvance.d.ts.map