/**
 * Order of execution:
 * - Transfer
 * - Change
 * - Add
 * - Remove
 *
 * Should allow the most flexibility whenever combination of migrations is necessary (e.g. Transfer + Change)
 */
export declare function migrateIfNecessary(): void;
//# sourceMappingURL=settingsMigration.d.ts.map