import type { ModernExtend } from "./types";
declare const extend: {
    dimmerLoadControlMode: () => ModernExtend;
    dimmerSwitchMode: () => ModernExtend;
    dimmerInvertSwitch: () => ModernExtend;
    dimmerSceneActivation: () => ModernExtend;
    dimmerS1DoubleClickScene: () => ModernExtend;
    dimmerS2DoubleClickScene: () => ModernExtend;
    dimmerMinBrightnessLevel: () => ModernExtend;
    dimmerMaxBrightnessLevel: () => ModernExtend;
    dimmerManualDimmingStepSize: () => ModernExtend;
    dimmerManualDimmingTime: () => ModernExtend;
};
export { extend };
//# sourceMappingURL=adurosmart.d.ts.map