import type { DefinitionWithExtend } from "../lib/types";
export interface SonoffEwelink {
    attributes: {
        networkLed: number;
        backLight: number;
        faultCode: number;
        radioPower: number;
        radioPowerWithManuCode: number;
        delayedPowerOnState: number;
        delayedPowerOnTime: number;
        externalTriggerMode: number;
        detachRelayMode: number;
        deviceWorkMode: number;
        detachRelayMode2: number;
        motorTravelCalibrationAction: number;
        lackWaterCloseValveTimeout: number;
        motorTravelCalibrationStatus: number;
        motorRunStatus: number;
        acCurrentCurrentValue: number;
        acCurrentVoltageValue: number;
        acCurrentPowerValue: number;
        outlet_control_protect: number;
        energyToday: number;
        energyMonth: number;
        energyYesterday: number;
        setCalibrationAction: number[];
        calibrationStatus: number;
        calibrationProgress: number;
        minBrightnessThreshold: number;
        transitionTime: number;
        dimmingLightRate: number;
        programmableStepperSequence: number[];
    };
    commands: {
        protocolData: {
            data: number[];
        };
    };
    commandResponses: never;
}
export declare const definitions: DefinitionWithExtend[];
//# sourceMappingURL=sonoff.d.ts.map