import{aU as c}from"./index-VJpwYvBU.js";var f,y;function A(){if(y)return f;y=1;var r=typeof c=="object"&&c&&c.Object===Object&&c;return f=r,f}var b,d;function k(){if(d)return b;d=1;var r=A(),e=typeof self=="object"&&self&&self.Object===Object&&self,a=r||e||Function("return this")();return b=a,b}var s,S;function h(){if(S)return s;S=1;var r=k(),e=r.Symbol;return s=e,s}var g,_;function x(){if(_)return g;_=1;var r=Array.isArray;return g=r,g}var l,q;function I(){if(q)return l;q=1;var r=h(),e=Object.prototype,a=e.hasOwnProperty,i=e.toString,n=r?r.toStringTag:void 0;function u(t){var o=a.call(t,n),p=t[n];try{t[n]=void 0;var G=!0}catch{}var w=i.call(t);return G&&(o?t[n]=p:delete t[n]),w}return l=u,l}var v,O;function L(){if(O)return v;O=1;var r=Object.prototype,e=r.toString;function a(i){return e.call(i)}return v=a,v}var j,m;function F(){if(m)return j;m=1;var r=h(),e=I(),a=L(),i="[object Null]",n="[object Undefined]",u=r?r.toStringTag:void 0;function t(o){return o==null?o===void 0?n:i:u&&u in Object(o)?e(o):a(o)}return j=t,j}var T,R;function N(){if(R)return T;R=1;function r(e){return e!=null&&typeof e=="object"}return T=r,T}export{k as a,N as b,A as c,x as d,h as e,F as r};
