import { Status } from "./definition/status";
export declare class ZclStatusError extends Error {
    code: Status;
    constructor(code: Status, details?: string);
}
//# sourceMappingURL=zclStatusError.d.ts.map