"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Buffalo = void 0;
var buffalo_1 = require("./buffalo");
Object.defineProperty(exports, "Buffalo", { enumerable: true, get: function () { return buffalo_1.Buffalo; } });
//# sourceMappingURL=index.js.map