import { TZDate } from "../date/index.js";
/**
 * The function creates accepts a time zone and returns a function that creates
 * a new `TZDate` instance in the time zone from the provided value. Use it to
 * provide the context for the date-fns functions, via the `in` option.
 *
 * @param timeZone - Time zone name (IANA or UTC offset)
 *
 * @returns Function that creates a new `TZDate` instance in the time zone
 */
export declare const tz: (timeZone: string) => (value: Date | number | string) => TZDate;
//# sourceMappingURL=index.d.ts.map