import{r as d,j as e}from"./index-VJpwYvBU.js";const n=d.memo(s=>{const{label:l,detail:a,onChange:t,children:r,...c}=s;return e.jsxs("fieldset",{className:"fieldset",children:[l&&e.jsxs("legend",{className:"fieldset-legend",children:[l,s.required?" *":""]}),e.jsx("select",{className:`select ${s.required?"validator":""}`,onChange:t,...c,children:r}),a&&e.jsx("span",{className:"label",children:a})]})});export{n as S};
