import { TWorkerDefinition } from 'worker-factory';
import { IWorkerTimersWorkerCustomDefinition } from '../interfaces';
export type TWorkerTimersWorkerDefinition = TWorkerDefinition<IWorkerTimersWorkerCustomDefinition>;
//# sourceMappingURL=worker-timers-worker-definition.d.ts.map