import{a as d,I as t,j as e}from"./index-VJpwYvBU.js";import{A as i,R as m}from"./RecallRemove-B8kK-jlT.js";import"./envs-CRUy-_iG.js";import"./DashboardFeatureWrapper-C5cTX2Yd.js";import"./startCase-DNIChZAc.js";import"./DisplayValue-BX0bcoSg.js";import"./isObject--vsEa_js.js";import"./_createCompounder-JrZUBxnL.js";import"./isObjectLike-CKrHmw-v.js";import"./index-C1LTvZEM.js";import"./InputField-B_6Ee4_m.js";import"./extends-CF3RwP-h.js";import"./react-select.esm-DM_HiCZr.js";import"./defineProperty-CegpTSss.js";import"./floating-ui.dom-DbzrgdRQ.js";function R({sourceIdx:r,device:a}){const o=d(t(s=>s.deviceStates[r][a.friendly_name]));return e.jsxs("div",{className:"flex flex-row flex-wrap gap-4 w-full",children:[e.jsx("div",{className:"card card-border bg-base-200 border-base-300 rounded-box shadow-md flex-1",children:e.jsx("div",{className:"card-body p-4",children:e.jsx(i,{sourceIdx:r,target:a,deviceState:o??{}})})}),e.jsx("div",{className:"card card-border bg-base-200 border-base-300 rounded-box shadow-md flex-1",children:e.jsx("div",{className:"card-body p-4",children:e.jsx(m,{sourceIdx:r,target:a})})})]})}export{R as default};
