import { TWorkerTimersBrokerLoader, TWorkerTimersBrokerWrapper } from './types';
export * from './interfaces/index';
export * from './types/index';
export declare const wrap: TWorkerTimersBrokerWrapper;
export declare const load: TWorkerTimersBrokerLoader;
//# sourceMappingURL=module.d.ts.map