import type { TClusterAttributeKeys } from "zigbee-herdsman/dist/zspec/zcl/definition/clusters-types";
import type { Tz, Zh } from "./types";
export declare function readColorCapabilities(endpoint: Zh.Endpoint): Promise<void>;
export declare function readColorTempMinMax(endpoint: Zh.Endpoint): Promise<void>;
export declare function readColorAttributes(entity: Zh.Endpoint | Zh.Group, meta: Tz.Meta, additionalAttributes?: TClusterAttributeKeys<"lightingColorCtrl">): ("remainingTime" | "options" | "currentHue" | "currentSaturation" | "currentX" | "currentY" | "driftCompensation" | "compensationText" | "colorTemperature" | "colorMode" | "numPrimaries" | "primary1X" | "primary1Y" | "primary1Intensity" | "primary2X" | "primary2Y" | "primary2Intensity" | "primary3X" | "primary3Y" | "primary3Intensity" | "primary4X" | "primary4Y" | "primary4Intensity" | "primary5X" | "primary5Y" | "primary5Intensity" | "primary6X" | "primary6Y" | "primary6Intensity" | "whitePointX" | "whitePointY" | "colorPointRX" | "colorPointRY" | "colorPointRIntensity" | "colorPointGX" | "colorPointGY" | "colorPointGIntensity" | "colorPointBX" | "colorPointBY" | "colorPointBIntensity" | "enhancedCurrentHue" | "enhancedColorMode" | "colorLoopActive" | "colorLoopDirection" | "colorLoopTime" | "colorLoopStartEnhancedHue" | "colorLoopStoredEnhancedHue" | "colorCapabilities" | "colorTempPhysicalMin" | "colorTempPhysicalMax" | "coupleColorTempToLevelMin" | "startUpColorTemperature" | "tuyaRgbMode" | "tuyaBrightness")[];
export declare function findColorTempRange(entity: Zh.Endpoint | Zh.Group): number[];
export declare function clampColorTemp(colorTemp: number, colorTempMin: number, colorTempMax: number): number;
export declare function configure(device: Zh.Device, coordinatorEndpoint: Zh.Endpoint, readColorTempMinMaxAttribute: boolean): Promise<void>;
//# sourceMappingURL=light.d.ts.map