/**
 * width=16 poly=0x1021 init=0x0000 refin=true refout=true xorout=0x0000 check=0x2189 residue=0x0000 name="CRC-16/KERMIT"
 */
export declare function crc16(data: Buffer): number;
/**
 * width=8 poly=0x4d init=0xff refin=true refout=true xorout=0xff check=0xd8 name="CRC-8/KOOP"
 */
export declare function crc8(data: Buffer): number;
//# sourceMappingURL=utils.d.ts.map