import type { TClusterAttributes } from "../zspec/zcl/definition/clusters-types";
export declare function timestampToZigbeeUtcTime(timestamp: number): number;
export declare function getTimeClusterAttributes(): TClusterAttributes<"genTime">;
/** used by tests */
export declare function clearCachedTimeData(): void;
//# sourceMappingURL=timeService.d.ts.map