import{r as m,u as i,ba as w,j as o,L as c,b1 as p,F as u,bb as b}from"./index-VJpwYvBU.js";const x=m.memo(({modelId:a,supported:e,definitionModel:s,icon:l=!1})=>{const{t:r}=i("zigbee");let n=a||r(k=>k.unknown),t=p;return e&&s&&(t=`https://www.zigbee2mqtt.io/devices/${encodeURIComponent(w(s))}.html`,n=s),o.jsxs(c,{target:"_blank",rel:"noopener noreferrer",to:t,className:"link link-hover",children:[n,l?o.jsx(u,{icon:b,className:"ms-0.5"}):null]})}),g=m.memo(({supported:a,definitionVendor:e,icon:s=!1})=>{const{t:l}=i("zigbee");let r=l(t=>t.unsupported),n=p;return a&&e&&(n=`https://www.zigbee2mqtt.io/supported-devices/#v=${encodeURIComponent(e)}`,r=e),o.jsxs(c,{target:"_blank",rel:"noopener noreferrer",to:n,className:"link link-hover",children:[r,s?o.jsx(u,{icon:b,className:"ms-0.5"}):null]})});export{x as D,g as V};
