import{aV as u}from"./index-VJpwYvBU.js";import{r as C}from"./_createCompounder-JrZUBxnL.js";var e,r;function i(){if(r)return e;r=1;var a=C(),s=a(function(o,n,t){return o+(t?"_":"")+n.toLowerCase()});return e=s,e}var m=i();const k=u(m);export{k as s};
