import{r,j as s,F as m}from"./index-VJpwYvBU.js";import{a as n}from"./index-C1LTvZEM.js";import{D as e}from"./DisplayValue-BX0bcoSg.js";const x=r.memo(({value:o})=>{const[t,a]=n(o);return s.jsxs(s.Fragment,{children:[s.jsx(m,{icon:t,className:a})," ",s.jsx(e,{value:o,name:"linkquality"})]})});export{x as L};
