import{r as i,j as e}from"./index-VJpwYvBU.js";const x=i.memo(a=>{const{label:s,detail:l,onChange:r,onBlur:t,...d}=a;return e.jsxs("fieldset",{className:"fieldset",children:[s&&e.jsxs("legend",{className:"fieldset-legend",children:[s,a.required?" *":""]}),e.jsx("textarea",{className:`textarea ${a.required?"validator":""}`,onChange:r,onBlur:t,...d}),l&&e.jsx("div",{className:"label",children:l})]})});export{x as T};
