import type { createCache } from './cache';
export declare const createGenerateUniqueNumber: (cache: ReturnType<typeof createCache>, lastNumberWeakMap: WeakMap<Map<number, any> | Set<number>, number>) => (collection: Map<number, any> | Set<number>) => number;
//# sourceMappingURL=generate-unique-number.d.ts.map