import Extension from "./extension";
export default class Health extends Extension {
    #private;
    start(): Promise<void>;
    stop(): Promise<void>;
    clearStats(): void;
}
//# sourceMappingURL=health.d.ts.map