"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const acova_1 = require("./acova");
const acuity_brands_lighting_1 = require("./acuity_brands_lighting");
const adeo_1 = require("./adeo");
const adurosmart_1 = require("./adurosmart");
const aeotec_1 = require("./aeotec");
const airam_1 = require("./airam");
const airzone_aidoo_1 = require("./airzone_aidoo");
const ajax_online_1 = require("./ajax_online");
const akuvox_1 = require("./akuvox");
const alchemy_1 = require("./alchemy");
const aldi_1 = require("./aldi");
const alecto_1 = require("./alecto");
const amina_1 = require("./amina");
const anchor_1 = require("./anchor");
const atlantic_1 = require("./atlantic");
const atsmart_1 = require("./atsmart");
const aubess_1 = require("./aubess");
const aurora_lighting_1 = require("./aurora_lighting");
const automaton_1 = require("./automaton");
const avatto_1 = require("./avatto");
const awox_1 = require("./awox");
const axis_1 = require("./axis");
const bacchus_1 = require("./bacchus");
const bankamp_1 = require("./bankamp");
const bega_1 = require("./bega");
const belkin_1 = require("./belkin");
const bitron_1 = require("./bitron");
const bituo_technik_1 = require("./bituo_technik");
const blaupunkt_1 = require("./blaupunkt");
const blitzwolf_1 = require("./blitzwolf");
const bosch_1 = require("./bosch");
const bouffalo_lab_1 = require("./bouffalo_lab");
const box_1 = require("./box");
const brimate_1 = require("./brimate");
const brun_holding_1 = require("./brun_holding");
const bseed_1 = require("./bseed");
const bticino_1 = require("./bticino");
const bubendorff_1 = require("./bubendorff");
const busch_jaeger_1 = require("./busch_jaeger");
const byun_1 = require("./byun");
const calex_1 = require("./calex");
const candeo_1 = require("./candeo");
const casaia_1 = require("./casaia");
const cel_1 = require("./cel");
const centralite_1 = require("./centralite");
const chacon_1 = require("./chacon");
const cigol_1 = require("./cigol");
const cleode_1 = require("./cleode");
const cleverio_1 = require("./cleverio");
const climax_1 = require("./climax");
const commercial_electric_1 = require("./commercial_electric");
const cree_1 = require("./cree");
const ctm_1 = require("./ctm");
const current_products_corp_1 = require("./current_products_corp");
const custom_devices_diy_1 = require("./custom_devices_diy");
const cwd_1 = require("./cwd");
const cy_lighting_1 = require("./cy_lighting");
const danalock_1 = require("./danalock");
const danfoss_1 = require("./danfoss");
const databyte_1 = require("./databyte");
const datek_1 = require("./datek");
const dawon_dns_1 = require("./dawon_dns");
const develco_1 = require("./develco");
const digi_1 = require("./digi");
const direct_signs_1 = require("./direct_signs");
const diyruz_1 = require("./diyruz");
const dlink_1 = require("./dlink");
const dnake_1 = require("./dnake");
const domraem_1 = require("./domraem");
const dowsing_reynolds_1 = require("./dowsing_reynolds");
const dqsmart_1 = require("./dqsmart");
const dresden_elektronik_1 = require("./dresden_elektronik");
const easyaccess_1 = require("./easyaccess");
const easyiot_1 = require("./easyiot");
const eatonhalo_led_1 = require("./eatonhalo_led");
const echostar_1 = require("./echostar");
const ecodim_1 = require("./ecodim");
const ecolink_1 = require("./ecolink");
const ecosmart_1 = require("./ecosmart");
const ecozy_1 = require("./ecozy");
const edp_1 = require("./edp");
const efekta_1 = require("./efekta");
const eglo_1 = require("./eglo");
const elko_1 = require("./elko");
const enbrighten_1 = require("./enbrighten");
const engo_1 = require("./engo");
const enkin_1 = require("./enkin");
const enocean_1 = require("./enocean");
const envilar_1 = require("./envilar");
const espressif_1 = require("./espressif");
const essentialb_1 = require("./essentialb");
const essentials_1 = require("./essentials");
const eucontrols_1 = require("./eucontrols");
const eurotronic_1 = require("./eurotronic");
const evanell_1 = require("./evanell");
const evn_1 = require("./evn");
const evology_1 = require("./evology");
const evvr_1 = require("./evvr");
const ewelink_1 = require("./ewelink");
const ezex_1 = require("./ezex");
const ezviz_1 = require("./ezviz");
const fantem_1 = require("./fantem");
const feibit_1 = require("./feibit");
const fireangel_1 = require("./fireangel");
const frankever_1 = require("./frankever");
const frient_1 = require("./frient");
const futurehome_1 = require("./futurehome");
const ge_1 = require("./ge");
const gewiss_1 = require("./gewiss");
const gidealed_1 = require("./gidealed");
const giderwel_1 = require("./giderwel");
const giex_1 = require("./giex");
const girier_1 = require("./girier");
const gledopto_1 = require("./gledopto");
const gmmts_1 = require("./gmmts");
const gmy_1 = require("./gmy");
const gs_1 = require("./gs");
const gumax_1 = require("./gumax");
const halemeier_1 = require("./halemeier");
const halo_smart_labs_1 = require("./halo_smart_labs");
const hampton_bay_1 = require("./hampton_bay");
const heatit_1 = require("./heatit");
const heiman_1 = require("./heiman");
const heimgard_technologies_1 = require("./heimgard_technologies");
const hej_1 = require("./hej");
const hfh_1 = require("./hfh");
const hilux_1 = require("./hilux");
const hive_1 = require("./hive");
const hoftronic_1 = require("./hoftronic");
const homeseer_1 = require("./homeseer");
const hommyn_1 = require("./hommyn");
const honyar_1 = require("./honyar");
const hornbach_1 = require("./hornbach");
const hzc_electric_1 = require("./hzc_electric");
const ITCommander_1 = require("./ITCommander");
const icasa_1 = require("./icasa");
const idinio_1 = require("./idinio");
const ihorn_1 = require("./ihorn");
const ikea_1 = require("./ikea");
const ilightsin_1 = require("./ilightsin");
const iluminize_1 = require("./iluminize");
const ilux_1 = require("./ilux");
const imhotepcreation_1 = require("./imhotepcreation");
const immax_1 = require("./immax");
const imou_1 = require("./imou");
const innr_1 = require("./innr");
const inovelli_1 = require("./inovelli");
const insta_1 = require("./insta");
const intuis_1 = require("./intuis");
const iolloi_1 = require("./iolloi");
const iotperfect_1 = require("./iotperfect");
const iris_1 = require("./iris");
const istar_1 = require("./istar");
const jasco_1 = require("./jasco");
const javis_1 = require("./javis");
const jethome_1 = require("./jethome");
const jiawen_1 = require("./jiawen");
const jumitech_1 = require("./jumitech");
const jxuan_1 = require("./jxuan");
const kami_1 = require("./kami");
const keen_home_1 = require("./keen_home");
const klikaanklikuit_1 = require("./klikaanklikuit");
const kmpcil_1 = require("./kmpcil");
const konke_1 = require("./konke");
const ksentry_1 = require("./ksentry");
const kurvia_1 = require("./kurvia");
const kwikset_1 = require("./kwikset");
const lanesto_1 = require("./lanesto");
const lds_1 = require("./lds");
const led_trading_1 = require("./led_trading");
const ledvance_1 = require("./ledvance");
const leedarson_1 = require("./leedarson");
const legrand_1 = require("./legrand");
const lellki_1 = require("./lellki");
const letsled_1 = require("./letsled");
const letv_1 = require("./letv");
const leviton_1 = require("./leviton");
const lg_1 = require("./lg");
const lidl_1 = require("./lidl");
const lifecontrol_1 = require("./lifecontrol");
const lightsolutions_1 = require("./lightsolutions");
const lincukoo_1 = require("./lincukoo");
const linkind_1 = require("./linkind");
const linptech_1 = require("./linptech");
const livingwise_1 = require("./livingwise");
const livolo_1 = require("./livolo");
const lixee_1 = require("./lixee");
const lonsonho_1 = require("./lonsonho");
const ls_1 = require("./ls");
const lubeez_1 = require("./lubeez");
const lumi_1 = require("./lumi");
const lupus_1 = require("./lupus");
const lutron_1 = require("./lutron");
const lux_1 = require("./lux");
const lytko_1 = require("./lytko");
const m_elec_1 = require("./m_elec");
const makegood_1 = require("./makegood");
const matcall_bv_1 = require("./matcall_bv");
const mazda_1 = require("./mazda");
const meazon_1 = require("./meazon");
const mercator_1 = require("./mercator");
const miboxer_1 = require("./miboxer");
const micromatic_1 = require("./micromatic");
const mill_1 = require("./mill");
const mindy_1 = require("./mindy");
const modular_1 = require("./modular");
const moes_1 = require("./moes");
const msh_1 = require("./msh");
const muller_licht_1 = require("./muller_licht");
const multir_1 = require("./multir");
const multiterm_1 = require("./multiterm");
const namron_1 = require("./namron");
const nanoleaf_1 = require("./nanoleaf");
const nedis_1 = require("./nedis");
const neo_1 = require("./neo");
const net2grid_1 = require("./net2grid");
const netica_1 = require("./netica");
const netvox_1 = require("./netvox");
const nexelec_1 = require("./nexelec");
const niko_1 = require("./niko");
const ninja_blocks_1 = require("./ninja_blocks");
const niviss_1 = require("./niviss");
const nobo_1 = require("./nobo");
const nodon_1 = require("./nodon");
const nordtronic_1 = require("./nordtronic");
const nous_1 = require("./nous");
const novo_1 = require("./novo");
const nue_3a_1 = require("./nue_3a");
const nyce_1 = require("./nyce");
const onenuo_1 = require("./onenuo");
const onesti_1 = require("./onesti");
const onokom_1 = require("./onokom");
const openlumi_1 = require("./openlumi");
const orvibo_1 = require("./orvibo");
const orztech_1 = require("./orztech");
const osram_1 = require("./osram");
const oujiabao_1 = require("./oujiabao");
const owon_1 = require("./owon");
const ozsmartthings_1 = require("./ozsmartthings");
const paul_neuhaus_1 = require("./paul_neuhaus");
const paulmann_1 = require("./paulmann");
const peq_1 = require("./peq");
const perenio_1 = require("./perenio");
const philio_1 = require("./philio");
const philips_1 = require("./philips");
const plaid_1 = require("./plaid");
const plugwise_1 = require("./plugwise");
const profalux_1 = require("./profalux");
const prolight_1 = require("./prolight");
const purmo_1 = require("./purmo");
const pushok_1 = require("./pushok");
const qa_1 = require("./qa");
const qmotion_1 = require("./qmotion");
const qoto_1 = require("./qoto");
const quirky_1 = require("./quirky");
const quotra_1 = require("./quotra");
const rademacher_1 = require("./rademacher");
const radium_1 = require("./radium");
const raex_1 = require("./raex");
const repenic_ltd_1 = require("./repenic_ltd");
const rgb_genie_1 = require("./rgb_genie");
const robb_1 = require("./robb");
const roome_1 = require("./roome");
const rtx_1 = require("./rtx");
const salus_controls_1 = require("./salus_controls");
const samotech_1 = require("./samotech");
const saswell_1 = require("./saswell");
const sber_1 = require("./sber");
const scanproducts_1 = require("./scanproducts");
const schlage_1 = require("./schlage");
const schneider_electric_1 = require("./schneider_electric");
const schwaiger_1 = require("./schwaiger");
const seastar_intelligence_1 = require("./seastar_intelligence");
const securifi_1 = require("./securifi");
const sengled_1 = require("./sengled");
const senoro_1 = require("./senoro");
const sercomm_1 = require("./sercomm");
const shade_control_1 = require("./shade_control");
const shelly_1 = require("./shelly");
const shenzhen_homa_1 = require("./shenzhen_homa");
const shinasystem_1 = require("./shinasystem");
const shyugj_1 = require("./shyugj");
const siglis_1 = require("./siglis");
const sikom_1 = require("./sikom");
const silicon_labs_1 = require("./silicon_labs");
const simon_1 = require("./simon");
const simpla_home_1 = require("./simpla_home");
const sinope_1 = require("./sinope");
const siterwell_1 = require("./siterwell");
const skydance_1 = require("./skydance");
const slacky_diy_1 = require("./slacky_diy");
const slv_1 = require("./slv");
const smarli_1 = require("./smarli");
const smart_home_pty_1 = require("./smart_home_pty");
const smart9_1 = require("./smart9");
const smartenit_1 = require("./smartenit");
const smartthings_1 = require("./smartthings");
const smartwings_1 = require("./smartwings");
const smlight_1 = require("./smlight");
const soanalarm_1 = require("./soanalarm");
const sohan_electric_1 = require("./sohan_electric");
const solaredge_1 = require("./solaredge");
const soma_1 = require("./soma");
const somfy_1 = require("./somfy");
const somgoms_1 = require("./somgoms");
const sonoff_1 = require("./sonoff");
const sowilo_1 = require("./sowilo");
const spacetronik_1 = require("./spacetronik");
const spotmau_1 = require("./spotmau");
const sprut_1 = require("./sprut");
const stello_1 = require("./stello");
const stelpro_1 = require("./stelpro");
const sunricher_1 = require("./sunricher");
const superled_1 = require("./superled");
const swann_1 = require("./swann");
const sylvania_1 = require("./sylvania");
const tapestry_1 = require("./tapestry");
const task_lighting_1 = require("./task_lighting");
const tci_1 = require("./tci");
const tcl_1 = require("./tcl");
const tech_1 = require("./tech");
const technicolor_1 = require("./technicolor");
const terncy_1 = require("./terncy");
const the_light_group_1 = require("./the_light_group");
const third_reality_1 = require("./third_reality");
const tis_control_1 = require("./tis_control");
const titan_products_1 = require("./titan_products");
const tlwglobal_1 = require("./tlwglobal");
const tnce_1 = require("./tnce");
const tplink_1 = require("./tplink");
const trust_1 = require("./trust");
const tubeszb_1 = require("./tubeszb");
const tuya_1 = require("./tuya");
const ubisys_1 = require("./ubisys");
const uhome_1 = require("./uhome");
const universal_electronics_inc_1 = require("./universal_electronics_inc");
const vav_1 = require("./vav");
const vbled_1 = require("./vbled");
const vesternet_1 = require("./vesternet");
const viessmann_1 = require("./viessmann");
const villeroy_boch_1 = require("./villeroy_boch");
const vimar_1 = require("./vimar");
const visonic_1 = require("./visonic");
const vrey_1 = require("./vrey");
const vsmart_1 = require("./vsmart");
const wally_1 = require("./wally");
const waxman_1 = require("./waxman");
const weiser_1 = require("./weiser");
const weten_1 = require("./weten");
const wirenboard_1 = require("./wirenboard");
const wisdom_1 = require("./wisdom");
const wmun_1 = require("./wmun");
const woolley_1 = require("./woolley");
const woox_1 = require("./woox");
const wyze_1 = require("./wyze");
const xal_1 = require("./xal");
const xenon_1 = require("./xenon");
const xinghuoyuan_1 = require("./xinghuoyuan");
const xyzroe_1 = require("./xyzroe");
const yale_1 = require("./yale");
const yandex_1 = require("./yandex");
const ynoa_1 = require("./ynoa");
const yokis_1 = require("./yokis");
const yookee_1 = require("./yookee");
const ysrsai_1 = require("./ysrsai");
const zbeacon_1 = require("./zbeacon");
const zemismart_1 = require("./zemismart");
const zen_1 = require("./zen");
const zigbeetlc_1 = require("./zigbeetlc");
const zipato_1 = require("./zipato");
const zunzunbee_1 = require("./zunzunbee");
const definitions = [
    ...acova_1.definitions,
    ...acuity_brands_lighting_1.definitions,
    ...adeo_1.definitions,
    ...adurosmart_1.definitions,
    ...aeotec_1.definitions,
    ...airam_1.definitions,
    ...airzone_aidoo_1.definitions,
    ...ajax_online_1.definitions,
    ...akuvox_1.definitions,
    ...alchemy_1.definitions,
    ...aldi_1.definitions,
    ...alecto_1.definitions,
    ...amina_1.definitions,
    ...anchor_1.definitions,
    ...atlantic_1.definitions,
    ...atsmart_1.definitions,
    ...aubess_1.definitions,
    ...aurora_lighting_1.definitions,
    ...automaton_1.definitions,
    ...avatto_1.definitions,
    ...awox_1.definitions,
    ...axis_1.definitions,
    ...bacchus_1.definitions,
    ...bankamp_1.definitions,
    ...bega_1.definitions,
    ...belkin_1.definitions,
    ...bitron_1.definitions,
    ...bituo_technik_1.definitions,
    ...blaupunkt_1.definitions,
    ...blitzwolf_1.definitions,
    ...bosch_1.definitions,
    ...bouffalo_lab_1.definitions,
    ...box_1.definitions,
    ...brimate_1.definitions,
    ...brun_holding_1.definitions,
    ...bseed_1.definitions,
    ...bticino_1.definitions,
    ...bubendorff_1.definitions,
    ...busch_jaeger_1.definitions,
    ...byun_1.definitions,
    ...calex_1.definitions,
    ...candeo_1.definitions,
    ...casaia_1.definitions,
    ...cel_1.definitions,
    ...centralite_1.definitions,
    ...chacon_1.definitions,
    ...cigol_1.definitions,
    ...cleode_1.definitions,
    ...cleverio_1.definitions,
    ...climax_1.definitions,
    ...commercial_electric_1.definitions,
    ...cree_1.definitions,
    ...ctm_1.definitions,
    ...current_products_corp_1.definitions,
    ...custom_devices_diy_1.definitions,
    ...cwd_1.definitions,
    ...cy_lighting_1.definitions,
    ...danalock_1.definitions,
    ...danfoss_1.definitions,
    ...databyte_1.definitions,
    ...datek_1.definitions,
    ...dawon_dns_1.definitions,
    ...develco_1.definitions,
    ...digi_1.definitions,
    ...direct_signs_1.definitions,
    ...diyruz_1.definitions,
    ...dlink_1.definitions,
    ...dnake_1.definitions,
    ...domraem_1.definitions,
    ...dowsing_reynolds_1.definitions,
    ...dresden_elektronik_1.definitions,
    ...dqsmart_1.definitions,
    ...easyaccess_1.definitions,
    ...easyiot_1.definitions,
    ...eatonhalo_led_1.definitions,
    ...echostar_1.definitions,
    ...ecodim_1.definitions,
    ...ecolink_1.definitions,
    ...ecosmart_1.definitions,
    ...ecozy_1.definitions,
    ...edp_1.definitions,
    ...efekta_1.definitions,
    ...eglo_1.definitions,
    ...elko_1.definitions,
    ...enbrighten_1.definitions,
    ...enkin_1.definitions,
    ...enocean_1.definitions,
    ...envilar_1.definitions,
    ...espressif_1.definitions,
    ...essentialb_1.definitions,
    ...essentials_1.definitions,
    ...eucontrols_1.definitions,
    ...eurotronic_1.definitions,
    ...evanell_1.definitions,
    ...evn_1.definitions,
    ...evology_1.definitions,
    ...evvr_1.definitions,
    ...ewelink_1.definitions,
    ...ezex_1.definitions,
    ...ezviz_1.definitions,
    ...fantem_1.definitions,
    ...feibit_1.definitions,
    ...fireangel_1.definitions,
    ...frankever_1.definitions,
    ...frient_1.definitions,
    ...futurehome_1.definitions,
    ...ge_1.definitions,
    ...gewiss_1.definitions,
    ...gidealed_1.definitions,
    ...giderwel_1.definitions,
    ...giex_1.definitions,
    ...girier_1.definitions,
    ...gledopto_1.definitions,
    ...gmmts_1.definitions,
    ...gmy_1.definitions,
    ...gs_1.definitions,
    ...gumax_1.definitions,
    ...halemeier_1.definitions,
    ...halo_smart_labs_1.definitions,
    ...hampton_bay_1.definitions,
    ...heatit_1.definitions,
    ...heiman_1.definitions,
    ...heimgard_technologies_1.definitions,
    ...hej_1.definitions,
    ...hfh_1.definitions,
    ...hilux_1.definitions,
    ...hive_1.definitions,
    ...hoftronic_1.definitions,
    ...homeseer_1.definitions,
    ...hommyn_1.definitions,
    ...honyar_1.definitions,
    ...hornbach_1.definitions,
    ...hzc_electric_1.definitions,
    ...icasa_1.definitions,
    ...idinio_1.definitions,
    ...ihorn_1.definitions,
    ...ikea_1.definitions,
    ...ilightsin_1.definitions,
    ...iluminize_1.definitions,
    ...ilux_1.definitions,
    ...imhotepcreation_1.definitions,
    ...immax_1.definitions,
    ...imou_1.definitions,
    ...innr_1.definitions,
    ...inovelli_1.definitions,
    ...insta_1.definitions,
    ...intuis_1.definitions,
    ...iolloi_1.definitions,
    ...iotperfect_1.definitions,
    ...iris_1.definitions,
    ...istar_1.definitions,
    ...ITCommander_1.definitions,
    ...jasco_1.definitions,
    ...javis_1.definitions,
    ...jethome_1.definitions,
    ...jiawen_1.definitions,
    ...jumitech_1.definitions,
    ...jxuan_1.definitions,
    ...kami_1.definitions,
    ...keen_home_1.definitions,
    ...klikaanklikuit_1.definitions,
    ...kmpcil_1.definitions,
    ...konke_1.definitions,
    ...ksentry_1.definitions,
    ...kurvia_1.definitions,
    ...kwikset_1.definitions,
    ...lanesto_1.definitions,
    ...lds_1.definitions,
    ...led_trading_1.definitions,
    ...ledvance_1.definitions,
    ...leedarson_1.definitions,
    ...legrand_1.definitions,
    ...lellki_1.definitions,
    ...letsled_1.definitions,
    ...letv_1.definitions,
    ...leviton_1.definitions,
    ...lg_1.definitions,
    ...lidl_1.definitions,
    ...lifecontrol_1.definitions,
    ...lightsolutions_1.definitions,
    ...lincukoo_1.definitions,
    ...linkind_1.definitions,
    ...linptech_1.definitions,
    ...livingwise_1.definitions,
    ...livolo_1.definitions,
    ...lixee_1.definitions,
    ...lonsonho_1.definitions,
    ...ls_1.definitions,
    ...lubeez_1.definitions,
    ...lupus_1.definitions,
    ...lutron_1.definitions,
    ...lux_1.definitions,
    ...lytko_1.definitions,
    ...m_elec_1.definitions,
    ...makegood_1.definitions,
    ...matcall_bv_1.definitions,
    ...mazda_1.definitions,
    ...meazon_1.definitions,
    ...mercator_1.definitions,
    ...miboxer_1.definitions,
    ...micromatic_1.definitions,
    ...mill_1.definitions,
    ...mindy_1.definitions,
    ...modular_1.definitions,
    ...moes_1.definitions,
    ...msh_1.definitions,
    ...muller_licht_1.definitions,
    ...multir_1.definitions,
    ...multiterm_1.definitions,
    ...namron_1.definitions,
    ...nanoleaf_1.definitions,
    ...neo_1.definitions,
    ...nedis_1.definitions,
    ...net2grid_1.definitions,
    ...netica_1.definitions,
    ...netvox_1.definitions,
    ...nexelec_1.definitions,
    ...niko_1.definitions,
    ...ninja_blocks_1.definitions,
    ...niviss_1.definitions,
    ...nodon_1.definitions,
    ...nordtronic_1.definitions,
    ...nous_1.definitions,
    ...nobo_1.definitions,
    ...novo_1.definitions,
    ...nue_3a_1.definitions,
    ...nyce_1.definitions,
    ...onenuo_1.definitions,
    ...onesti_1.definitions,
    ...onokom_1.definitions,
    ...openlumi_1.definitions,
    ...orvibo_1.definitions,
    ...orztech_1.definitions,
    ...osram_1.definitions,
    ...oujiabao_1.definitions,
    ...owon_1.definitions,
    ...ozsmartthings_1.definitions,
    ...paulmann_1.definitions,
    ...paul_neuhaus_1.definitions,
    ...peq_1.definitions,
    ...perenio_1.definitions,
    ...philio_1.definitions,
    ...philips_1.definitions,
    ...plaid_1.definitions,
    ...plugwise_1.definitions,
    ...profalux_1.definitions,
    ...prolight_1.definitions,
    ...purmo_1.definitions,
    ...pushok_1.definitions,
    ...qmotion_1.definitions,
    ...qoto_1.definitions,
    ...quirky_1.definitions,
    ...quotra_1.definitions,
    ...rademacher_1.definitions,
    ...radium_1.definitions,
    ...raex_1.definitions,
    ...repenic_ltd_1.definitions,
    ...rgb_genie_1.definitions,
    ...robb_1.definitions,
    ...roome_1.definitions,
    ...lumi_1.definitions,
    ...rtx_1.definitions,
    ...salus_controls_1.definitions,
    ...samotech_1.definitions,
    ...saswell_1.definitions,
    ...sber_1.definitions,
    ...scanproducts_1.definitions,
    ...schlage_1.definitions,
    ...schneider_electric_1.definitions,
    ...schwaiger_1.definitions,
    ...seastar_intelligence_1.definitions,
    ...securifi_1.definitions,
    ...sengled_1.definitions,
    ...senoro_1.definitions,
    ...sercomm_1.definitions,
    ...shade_control_1.definitions,
    ...shelly_1.definitions,
    ...shenzhen_homa_1.definitions,
    ...shinasystem_1.definitions,
    ...shyugj_1.definitions,
    ...siglis_1.definitions,
    ...silicon_labs_1.definitions,
    ...sikom_1.definitions,
    ...simon_1.definitions,
    ...simpla_home_1.definitions,
    ...sinope_1.definitions,
    ...siterwell_1.definitions,
    ...skydance_1.definitions,
    ...slacky_diy_1.definitions,
    ...slv_1.definitions,
    ...smarli_1.definitions,
    ...smart9_1.definitions,
    ...smartenit_1.definitions,
    ...smart_home_pty_1.definitions,
    ...smartthings_1.definitions,
    ...smartwings_1.definitions,
    ...smlight_1.definitions,
    ...soanalarm_1.definitions,
    ...sohan_electric_1.definitions,
    ...solaredge_1.definitions,
    ...soma_1.definitions,
    ...somfy_1.definitions,
    ...somgoms_1.definitions,
    ...sonoff_1.definitions,
    ...sowilo_1.definitions,
    ...spacetronik_1.definitions,
    ...spotmau_1.definitions,
    ...sprut_1.definitions,
    ...stello_1.definitions,
    ...stelpro_1.definitions,
    ...sunricher_1.definitions,
    ...superled_1.definitions,
    ...swann_1.definitions,
    ...sylvania_1.definitions,
    ...task_lighting_1.definitions,
    ...tapestry_1.definitions,
    ...tci_1.definitions,
    ...tcl_1.definitions,
    ...tech_1.definitions,
    ...technicolor_1.definitions,
    ...terncy_1.definitions,
    ...the_light_group_1.definitions,
    ...third_reality_1.definitions,
    ...tis_control_1.definitions,
    ...titan_products_1.definitions,
    ...tlwglobal_1.definitions,
    ...tnce_1.definitions,
    ...tplink_1.definitions,
    ...trust_1.definitions,
    ...tubeszb_1.definitions,
    ...tuya_1.definitions,
    ...ubisys_1.definitions,
    ...uhome_1.definitions,
    ...universal_electronics_inc_1.definitions,
    ...vav_1.definitions,
    ...vbled_1.definitions,
    ...vesternet_1.definitions,
    ...viessmann_1.definitions,
    ...villeroy_boch_1.definitions,
    ...vimar_1.definitions,
    ...visonic_1.definitions,
    ...vrey_1.definitions,
    ...wally_1.definitions,
    ...waxman_1.definitions,
    ...weiser_1.definitions,
    ...weten_1.definitions,
    ...wirenboard_1.definitions,
    ...wisdom_1.definitions,
    ...woolley_1.definitions,
    ...woox_1.definitions,
    ...wmun_1.definitions,
    ...wyze_1.definitions,
    ...xal_1.definitions,
    ...xenon_1.definitions,
    ...xinghuoyuan_1.definitions,
    ...xyzroe_1.definitions,
    ...yale_1.definitions,
    ...yandex_1.definitions,
    ...ynoa_1.definitions,
    ...yokis_1.definitions,
    ...yookee_1.definitions,
    ...ysrsai_1.definitions,
    ...zbeacon_1.definitions,
    ...zemismart_1.definitions,
    ...zen_1.definitions,
    ...zigbeetlc_1.definitions,
    ...zipato_1.definitions,
    ...zunzunbee_1.definitions,
    ...qa_1.definitions,
    ...engo_1.definitions,
    ...vsmart_1.definitions,
];
exports.default = definitions;
//# sourceMappingURL=index.js.map