import { TWorkerDefinition } from 'worker-factory';
import { IWorkerTimersWorkerCustomDefinition } from '../interfaces';

export type TWorkerTimersWorkerDefinition = TWorkerDefinition<IWorkerTimersWorkerCustomDefinition>;
