import{r as o,u as c,j as r}from"./index-VJpwYvBU.js";import{T as i}from"./TimeAgo-CeXmHrG4.js";const u=(e,t)=>{if(e)switch(t){case"ISO_8601":case"ISO_8601_local":return new Date(Date.parse(e));case"epoch":return new Date(e);case"disable":return;default:console.error(`Unknown last_seen type ${t}`);return}},p=o.memo(({lastSeen:e,config:t,fallback:s=null})=>{const{i18n:n}=c(),a=u(e,t);return a?r.jsx("span",{children:r.jsx(i,{datetime:a,locale:n.language})}):s});export{p as L};
