import{_ as v}from"./_getAllKeysIn-69c2a08d.js";import{aI as x,ay as u,ag as I,aM as l}from"./index-d7144f09.js";var m=v,p=x,K=u,e=I,O=l;function P(r,n,o,d){if(!e(r))return r;n=p(n,r);for(var i=-1,f=n.length,g=f-1,a=r;a!=null&&++i<f;){var s=O(n[i]),t=o;if(s==="__proto__"||s==="constructor"||s==="prototype")return r;if(i!=g){var _=a[s];t=d?d(_,s,a):void 0,t===void 0&&(t=e(_)?_:K(n[i+1])?[]:{})}m(a,s,t),a=a[s]}return r}var h=P;export{h as _};
