import type { DefinitionWithExtend } from "../lib/types";
export interface OrviboSwitchRewiringArgs {
    endpointNames: string[];
    endpoints: {
        [n: string]: number;
    };
    endpointIds: {
        [n: number]: string;
    };
}
export declare const definitions: DefinitionWithExtend[];
//# sourceMappingURL=orvibo.d.ts.map