const {
  emitForest,
  emitForestLines,
  parseFunctionRanges,
  parseOffsets,
} = require("./ascii");
const {
  cloneFunctionCov,
  cloneProcessCov,
  cloneScriptCov,
  cloneRangeCov,
} = require("./clone");
const {
  compareScriptCovs,
  compareFunctionCovs,
  compareRangeCovs,
} = require("./compare");
const {
  mergeFunctionCovs,
  mergeProcessCovs,
  mergeScriptCovs,
} = require("./merge");
const { RangeTree } = require("./range-tree");

module.exports = {
  emitForest,
  emitForestLines,
  parseFunctionRanges,
  parseOffsets,
  cloneFunctionCov,
  cloneProcessCov,
  cloneScriptCov,
  cloneRangeCov,
  compareScriptCovs,
  compareFunctionCovs,
  compareRangeCovs,
  mergeFunctionCovs,
  mergeProcessCovs,
  mergeScriptCovs,
  RangeTree,
};
