import * as exposes from "../lib/exposes";
import type { DefinitionWithExtend, Fz, KeyValue } from "../lib/types";
interface SamsungAccelerometer {
    attributes: {
        motionThresholdMultiplier: number;
        motionThreshold: number;
        acceleration: number;
        xAxis: number;
        yAxis: number;
        zAxis: number;
    };
    commands: never;
    commandResponses: never;
}
interface SmartThingsArrivalSensor {
    attributes: never;
    commands: never;
    commandResponses: {
        arrivalSensorNotify: Record<string, never>;
    };
}
export declare const smartthingsExtend: {
    addManuSpecificSamsungAccelerometerCluster: () => import("../lib/types").ModernExtend;
    addManuSpecificSmartThingsArrivalSensorCluster: () => import("../lib/types").ModernExtend;
};
export declare const fzLocal: {
    acceleration: {
        cluster: "manuSpecificSamsungAccelerometer";
        type: ["attributeReport", "readResponse"];
        convert: (model: import("../lib/types").Definition, msg: Fz.Message<"manuSpecificSamsungAccelerometer", SamsungAccelerometer, ["attributeReport", "readResponse"]>, publish: import("../lib/types").Publish, options: KeyValue, meta: Fz.Meta) => KeyValue;
    };
    PGC410EU_presence: {
        cluster: "manuSpecificSmartThingsArrivalSensor";
        type: "commandArrivalSensorNotify";
        options: exposes.Numeric[];
        convert: (model: import("../lib/types").Definition, msg: Fz.Message<"manuSpecificSmartThingsArrivalSensor", SmartThingsArrivalSensor, "commandArrivalSensorNotify">, publish: import("../lib/types").Publish, options: KeyValue, meta: Fz.Meta) => {
            presence: boolean;
        };
    };
};
export declare const definitions: DefinitionWithExtend[];
export {};
//# sourceMappingURL=smartthings.d.ts.map