export { VitestNodeSnapshotEnvironment as VitestSnapshotEnvironment } from './chunks/node.fjCdwEIl.js';
import '@vitest/snapshot/environment';
import './chunks/utils.XdZDrNZV.js';
import '@vitest/utils';
