export declare function isNumberArray(value: unknown): value is number[];
export declare function isNumberArrayOfLength(value: unknown, length: number): boolean;
export declare function assertString(input: unknown): asserts input is string;
export declare function isObjectEmpty(object: object): boolean;
//# sourceMappingURL=utils.d.ts.map